uses dos, crt, m_files;

const
  max_nodes = 255;

type
  t_node_ptr = ^t_node;
  t_node = record
    value: byte;
    b: array[0..1] of t_node_ptr;
  end;

  t_node_arr = array[0..max_nodes] of t_node;
  t_node_array2 = ^t_node_arr;

  t_node_ptrarr = array[0..max_nodes] of t_node_ptr;

  t_tree_ptr =^t_tree;
  t_tree = object
    root: t_node_ptr;

    constructor init;
    destructor done; virtual;
  end;

  t_code_tab_rec = record
    nbits: byte;
    code: word;
  end;
  t_code_table = array[0..255] of t_code_tab_rec;


  t_code_tab2_rec = record
    value: byte;
    nbits: byte;
    code: word;
  end;
  t_code_table2 = array[0..255] of t_code_tab2_rec;

{  t_source = object
    f: tfile;

    constructor init(var _f: tfile);
    destructor done; virtual;
  end;
}
  t_bit_buffer = object{(t_source)}
    f: tfile;

    buffer: array[0..1] of byte;
    buff_idx: byte;
    bit_idx: byte;

    constructor init(var _f: tfile);
    destructor done; virtual;
    procedure readbyte; virtual;
    procedure get_bit(var _bit: byte); virtual;
    procedure toggle_buffer; virtual;
  end;


var
  i: integer;

  cnt: word;
  n_nodes: word;

  code_tab: t_code_table;

  source_file, dest_file: tfile;

  bit_buffer: t_bit_buffer;

  tree: t_tree_ptr;

procedure add_to_tree(tree: t_tree_ptr; c_rec: t_code_tab2_rec);
var
  temp_node, a_node: t_node_ptr;

  i: word;
  b_idx: byte;
begin
  a_node:=tree^.root;

  for i:=c_rec.nbits-1 downto 0 do
  begin
    b_idx:=(c_rec.code and (1 shl i)) shr i;

    if a_node^.b[b_idx]=nil then
    begin
      new(temp_node);
      temp_node^.b[0]:=nil;
      temp_node^.b[1]:=nil;
      temp_node^.value:=0;

      a_node^.b[b_idx]:=temp_node;
    end;
    a_node:=a_node^.b[b_idx];
  end;
  a_node^.value:=c_rec.value;
end;

procedure read_code_table(tree: t_tree_ptr);
var
  error, result: word;
  c_rec: t_code_tab2_rec;
  n_rec: word;
begin
  error:=read_block(source_file, n_rec, sizeof(n_rec), result);
  for i:=1 to n_rec do
  begin
    error:=read_block(source_file, c_rec, sizeof(c_rec), result);
    check_error(error);

    add_to_tree(tree, c_rec);
  end;
end;


constructor t_tree.init;
var
  i, j: word;
begin
  new(root);
  root^.value:=0;
  root^.b[0]:=nil;
  root^.b[1]:=nil;
end;

destructor t_tree.done;
begin
end;


constructor t_bit_buffer.init(var _f: tfile);
begin
  f:=_f;

  buff_idx:=0;
end;

destructor t_bit_buffer.done;
begin
end;

procedure t_bit_buffer.readbyte;
var
  error: word;
  result: word;
begin
  error:=read_block(f, buffer[buff_idx], sizeof(buffer[buff_idx]), result);
  check_error(error);
end;

procedure t_bit_buffer.get_bit(var _bit: byte);
var
  bit: byte;
begin
  if bit_idx=0 then
  begin
    readbyte;
    bit_idx:=8;
  end;
  dec(bit_idx);
  bit:=(buffer[buff_idx] and (1 shl bit_idx)) shr bit_idx;
  _bit:=bit;
end;

procedure t_bit_buffer.toggle_buffer;
begin
  if buff_idx=0 then buff_idx:=1 else buff_idx:=0;
end;

procedure init;
var
  error: word;
begin
  error:=open_file(source_file, paramstr(1), readonly);
  check_error(error);

  error:=make_file(dest_file, paramstr(2));
  check_error(error);
end;

procedure deinit;
var
  error: word;
begin
  if source_file.open then
  begin
    error:=close_file(source_file);
    check_error(error);
  end;
end;

procedure decode(dest: tfile);
var
  buffer: array[0..2*1024-1] of byte;
  error: word;
  result: word;
  i, j: word;

  ctab_idx: word;

  pom_pos: longint;
  l: longint;

  temp_node: t_node_ptr;
  bit: byte;
begin

  error:=read_block(bit_buffer.f, l, sizeof(l), result);
  check_error(error);

  temp_node:=tree^.root;
  for i:=0 to l do
  begin
    bit_buffer.get_bit(bit);
    if temp_node^.b[bit]=nil then
    begin
      error:=write_block(dest, temp_node^.value, sizeof(temp_node^.value), result);

      temp_node:=tree^.root;
    end;

    temp_node:=temp_node^.b[bit];
  end;
end;


procedure write_help;
begin
  writeln;
  writeln('Huffmann decoder 1.00   by David Andrs');
  writeln;
  writeln('Usage:');
  writeln('   huffdec [input file] [output file]');
  writeln;
  writeln('   [input file] - name of file to decode');
  writeln('   [output file] - name of output file');
  writeln;
end;

begin
  if paramcount<2 then
  begin
    write_help;
  end else
      begin
        init;

        bit_buffer.init(source_file);

        new(tree, init);

        writeln('reading code table...');
        read_code_table(tree);

        writeln('decoding...');
        decode(dest_file);

        dispose(tree, done);

        bit_buffer.done;

        deinit;
      end;
end.