{ modul pro ovladani simulace }
{ David Andrs }

unit m_simul;

interface

uses
  m_ants, m_antsc, m_area;

function init_simulation(var ants: t_ant_ptr; ant_cnt: word;
  ant_way: t_create_way; ant_req: t_ant_req_ptr; ant_file: string;
  var area: t_area; wood_cnt: word; area_way: t_create_way;
  area_file: string): boolean;

procedure deinit_simulation(var ants: t_ant_ptr; var area: t_area);

procedure start_simulation(ants: t_ant_ptr; var area: t_area);

procedure pause_simulation;
{ pozastavi simulaci }
procedure resume_simulation;
{ obnovi beh zastavene simulace }

implementation

uses
  dos, crt, m_tools, m_tvga, m_timer, m_glob, m_dsktp, m_menu, m_soubor;

var
  old_int08: procedure;
  timer_running: boolean;

  ticks: longint;           { pocet tiku, ktere ubehly pri spusteni timeru }
  sync_tick: word;
  sync: boolean;

{$F+,S-}
procedure my_int08; interrupt;
begin
  if timer_running then
  begin
    if ticks>=sync_tick then
    begin
      sync:=TRUE;
    end;
    inc(ticks);
  end;
  port[$20]:=$20;
end;
{$F-,S+}

procedure install_timer;
{ nainstaluje vlastni obsluznou rutinu na vektor 08 }
begin
  getintvec($08, @old_int08);
  setintvec($08, @my_int08);
end;

procedure uninstall_timer;
{ odinstaluje obsluznou rutinu, nainstalovanou procedurou 'install_timer' }
begin
  setintvec($08, @old_int08);
end;

function init_simulation(var ants: t_ant_ptr; ant_cnt: word;
  ant_way: t_create_way; ant_req: t_ant_req_ptr; ant_file: string;
  var area: t_area; wood_cnt: word; area_way: t_create_way;
  area_file: string): boolean;
var
  ok: boolean;
begin
  ok:=TRUE;
  { vygeneruj plochu }
  ok:=ok and create_area(area, area_way, wood_cnt, area_file);
  if ok then
  begin
    { vygeneruj mravence }
    ok:=ok and create_ants(ant_cnt, ants, ant_way, ant_req, ant_file);
    if ok then
      set_ants_to_area(ants, area)
    else
      say_error('Chyba pri vytvareni mravencu');
  end else
      begin
        say_error('Chyba pri vytvareni plochy');
      end;

  if ok then
  begin
    timer_running:=FALSE;
    install_timer;
  end;

  init_simulation:=ok;
end;

procedure deinit_simulation(var ants: t_ant_ptr; var area: t_area);
begin
  timer_running:=FALSE;
  restore_timer;
  uninstall_timer;

  deinit_ants(ants);
  { zrusi struktury mravencu }
end;

procedure pause_simulation;
begin
  timer_running:=FALSE;
  restore_timer;

  uninstall_timer;
end;

procedure resume_simulation;
begin
  install_timer;
  if timer_rate<>0 then speed_timer(timer_rate);
  timer_running:=TRUE;
end;

procedure change_speed;
const
  { umisteni na plose }
  x = 53;
  y = 12;
var
  temp_speed: word;
  exit_loop: boolean;

  key: char;
begin
  exit_loop:=FALSE;
  temp_speed:=global_speed;

  write_text(x+09, y, 14, 12, ' '+i2s(temp_speed, 3)+' ');
  write_text(x+14, y, 14, 0, #18);
  repeat
    if keypressed then
    begin
      key:=readkey;
      case key of
        #27:
        begin
          exit_loop:=TRUE;
        end;

        #13:
        begin
          global_speed:=temp_speed;

          sync_tick:=30-((global_speed-1)*3);
          exit_loop:=TRUE;
        end;

        #0:
        begin
          key:=readkey;
          case key of
            #72: { dolu }
            begin
              if temp_speed<10 then
              begin
                inc(temp_speed);
                write_text(x+09, y, 14, 12, ' '+i2s(temp_speed, 3)+' ');
              end;
            end;

            #80: { nahoru }
            begin
              if temp_speed>1 then
              begin
                dec(temp_speed);
                write_text(x+09, y, 14, 12, ' '+i2s(temp_speed, 3)+' ');
              end;
            end;

          end; { case }
        end; { #0 }
      end; { case }
    end;
  until exit_loop;
  write_text(x+09, y, 7, 0, ' '+i2s(global_speed, 3)+'  ');
end;

procedure change_perf;
begin
  graph_on:=not(graph_on);
  if graph_on then
    menu_simulace[3]:=' Maximalni vykon   vyp <M>'
  else
    menu_simulace[3]:=' Maximalni vykon   zap <M>';
end;

function really_quit: boolean;
var
  ans_yesno: byte;
begin
  write_text(52, 30, 15, 0, 'Opravdu ukoncit ?');
  dej_menu(menu_noyes, ans_yesno);
  write_text(52, 30, 15, 0, '                 ');
  case ans_yesno of
    1: really_quit:=FALSE;
    2: really_quit:=TRUE;
  end; { case }
end;

procedure start_simulation(ants: t_ant_ptr; var area: t_area);
var
  key: char;

  exit_simulation: boolean;

  ans_simulace, ans_ulozit: byte;
begin
  exit_simulation:=FALSE;

  while keypressed do key:=readkey;
  { vyprazdni buffer klavesnice }
  key:=#0;

  draw_stat;
  draw_area(area);

  speed_timer(500);

  sync:=FALSE;
  sync_tick:=30;
  ant_idx:=1;

  graph_on:=FALSE;
  change_perf;

  timer_running:=TRUE;
  repeat
    if keypressed then
    begin
      key:=upcase(readkey);
      case key of
        ' ':
        begin
          pause_simulation;

          repeat
            dej_menu(menu_simulace, ans_simulace);
            case ans_simulace of
              1: { zpet do simulace }
              begin
              end;
              2: { zmena rychlosti }
              begin
                change_speed;
              end;
              3: { maximalni vykon }
              begin
                change_perf;
              end;
              4: { ulozit }
              begin
                dej_menu(menu_ulozit, ans_ulozit);
                dosouboru(area, ants, ans_ulozit);
              end;
              5:
              begin
                exit_simulation:=really_quit;
{                exit_simulation:=TRUE;}
              end;
            end; { case }
          until (ans_simulace in [1, 5]);

          resume_simulation;
        end;

        'M':
        begin
          change_perf;
{          if graph_on then
            draw_area(area);}
        end;

        'S':
        begin
          pause_simulation;
          change_speed;
          resume_simulation;
        end;

        'Q', #27:
        begin
{          really_quit;}
          exit_simulation:=really_quit;
        end;
      end; { case }
    end; { if }

    if ant_idx<=ant_count then
    begin
      control_ant(area, act_ant);
      act_ant^.speed:=act_ant^.speed - 1;
      correct_speed(act_ant^.speed);
      act_ant:=act_ant^.next;
      inc(ant_idx);
    end else
        if ant_idx>ant_count then
        begin
          if graph_on then
          begin
            if sync then
            begin
              ant_idx:=1;
              sync:=FALSE;
              ticks:=0;
            end;
          end else
              begin
                ant_idx:=1;
                sync:=FALSE;
                ticks:=0;
              end;
        end;
(*
    write_text(70, 49, 7, 0, i2s(ticks,5));

    textcolor(7);
    write_text(60, 5, 7, 0, 'ant_idx: '+i2s(ant_idx,2));

    textcolor(7);
    write_text(60, 6, 7, 0, 'graph: '+i2s(byte(graph_on),2));
*)
  until (exit_simulation);

end;

begin
end.
