program Fotbal; {Program Fotbal '99 - tym Stanek, Sip, Machaty}

uses App, Dialogs, Drivers, Memory, Menus, Objects, Views, Graph, UGrafika, U_Fotbal;

const
   cmNastaveni = 101;
   cmStart = 102;
   cmOProgramu = 103;
   hcHra = 1000;
   hcNastaveni = 1001;
   hcStart = 1002;
   hcKonec = 1003;
   hcOProgramu = 1004;
   hcInformace = 2001;
   hcNastaveniHry = 2002;

type
   TData = record {Zaznam pro dialog}
      Obsazeni: Word;
      Jmena: array [1..2] of string [24];
      Zacina, Tvar: Word;
      Zbytek: array [1..4] of string [3];
   end;
   TParametry = record {Skutecne parametry pro grafiku}
      Delka, Vyska, DelkaBrany: Byte;
      Jmeno1, Jmeno2: string;
      Obsazeni, Zacina, Tvar, Zpomaleni: Byte;
   end;
   PNadpis = ^TNadpis; {Objekt nadpisu}
   TNadpis = object (TStaticText)
      function GetPalette: PPalette; virtual;
   end;
   PStavovaLista = ^TStavovaLista; {Nova lista, ktera zobrazuje informace}
   TStavovaLista = object (TStatusLine)
      function Hint (AHelpCtx: Word): string; virtual;
   end;
   POkno = ^TOkno;
   TOkno = object (TDialog) {Novy dialog, ktery se automaticky centruje...}
      constructor Init (Delka, Vyska: Byte; Titulek: TTitleStr);
      destructor Done; virtual;
   end;
   TAplikace = object (TApplication) {Hlavni objekt}
      Data: TData;
      Nadpis: PNadpis;
      Parametry: TParametry;
      constructor Init; {Konstruktor - zavede nadpis, inicializuje hodnoty}
      procedure InitMenuBar; virtual; {Menu...}
      procedure InitStatusLine; virtual; {Linka...}
      procedure HandleEvent (var Event: TEvent); virtual; {Sprava udalosti}
      procedure Informace (Delka, Vyska: Byte; Titulek: TTitleStr; Zprava: string); {Sablona univerzalniho okna pro zpravu}
      procedure OProgramu; {Zobrazi dialog "O programu"}
      function VytvorNastaveni: POkno; {Vytvori dialog "Nastaveni hry"}
      function Test: Boolean; {Validace hodnot v okne}
      procedure Nastaveni; {Sprava dialogu "Nastaveni hry"}
      procedure NastavParametry; {Konverze dialogoveho zaznamu na smysluplne hodnoty}
      procedure Start; {Spusti grafiku...}
   end;

var Aplikace: TAplikace;

const
   CNadpis: string [1] = #47;
   Implicitni: TData = (Obsazeni: 0; Jmena: ('Honza', 'Moje PC'); Zacina: 0; Tvar: 0; Zbytek: ('11', '11', '3', '0'));

(* Nadpis *)
function TNadpis.GetPalette: PPalette;
begin
   GetPalette := @CNadpis;
end;

(* Stavova lista *)
function TStavovaLista.Hint (AHelpCtx: Word): string;
begin
   case AHelpCtx of
      hcHra: Hint := 'Nastaveni parametru, start hry, konec';
      hcNastaveni: Hint := 'Nastaveni parametru hry (obsazeni, tvar hriste...)';
      hcStart: Hint := 'Start hry';
      hcKonec: Hint := 'Konec programu Fotbal ''99';
      hcOProgramu: Hint := 'Jmena autoru programu';
      hcInformace: Hint := 'Stisknete klavesu Enter';
      hcNastaveniHry: Hint := 'Nastavte parametry hry';
   else Hint := '';
   end;
end;

(* Okno *)
constructor TOkno.Init (Delka, Vyska: Byte; Titulek: TTitleStr);
var R: TRect;
begin
   R.Assign (0, 0, Delka, Vyska);
   inherited Init (R, Titulek);
   Options := Options or ofCentered;
   DisableCommands ([cmMenu]);
end;

destructor TOkno.Done;
begin
   inherited Done;
   EnableCommands ([cmMenu]);
end;

(* Aplikace *)
constructor TAplikace.Init;
var R: TRect;
begin
   inherited Init;
   Data := Implicitni;
   GetExtent (R);
   R.B.Y := R.A.Y + 1;
   Nadpis := New (PNadpis, Init (R, ^C'***** Fotbal ''99 *****'));
   Insert (Nadpis);
   OProgramu;
end;

procedure TAplikace.InitMenuBar;
var R: TRect;
begin
   GetExtent (R);
   Inc (R.A.Y);
   R.B.Y := R.A.Y + 1;
   MenuBar := New (PMenuBar, Init (R, NewMenu (
      NewSubMenu ('~H~ra', hcHra, NewMenu (
         NewItem ('~N~astaveni...', 'F2', kbF2, cmNastaveni, hcNastaveni,
         NewLine (
         NewItem ('~S~tart', 'F9', kbF9, cmStart, hcStart,
         NewItem ('~K~onec', 'Alt+K', kbAltK, cmQuit, hcKonec,
         NewLine (
         NewItem ('~O~ programu...', '', kbNoKey, cmOProgramu, hcOProgramu,
         nil))))))
      ), nil)
   )));
end;

procedure TAplikace.InitStatusLine;
var R: TRect;
begin
   GetExtent (R);
   R.A.Y := R.B.Y - 1;
   StatusLine := New (PStavovaLista, Init (R,
      NewStatusDef (0, 0,
         NewStatusKey ('~F10~ Menu', kbF10, cmMenu,
         NewStatusKey ('~F2~ Nastaveni', kbF2, cmNastaveni,
         NewStatusKey ('~F9~ Start', kbF9, cmStart,
         NewStatusKey ('~Alt+K~ Konec', kbAltK, cmQuit,
         nil)))),
      NewStatusDef (1, $FFFF,
         NewStatusKey ('~F10~ Menu', kbF10, cmMenu,
         NewStatusKey ('', kbCtrlF5, cmResize,
         nil)),
      nil))
   ));
end;

procedure TAplikace.HandleEvent (var Event: TEvent);
begin
   inherited HandleEvent (Event);
   if Event.What = evCommand then case Event.Command of
      cmNastaveni: Nastaveni;
      cmStart: Start;
      cmOProgramu: OProgramu;
   else Exit;
   end;
   ClearEvent (Event);
end;

procedure TAplikace.Informace (Delka, Vyska: Byte; Titulek: TTitleStr; Zprava: string);
var
   Okno: POkno;
   R: TRect;
   Tlacitko: PButton;
begin
   Okno := New (POkno, Init (Delka, Vyska, Titulek));
   with Okno^ do begin
      HelpCtx := hcInformace;
      GetExtent (R);
      R.Grow (-3, -2);
      Dec (R.B.Y, 2);
      Insert (New (PStaticText, Init (R, Zprava)));
      R.Assign (0, Size.Y - 3, 10, Size.Y - 1);
      Tlacitko := New (PButton, Init (R, '~O~K', cmOK, bfNormal));
      with Tlacitko^ do Options := Options or ofCenterX;
      Insert (Tlacitko);
   end;
   ExecuteDialog (Okno, nil);
end;

procedure TAplikace.OProgramu;
begin
   Informace (37, 14, 'O programu', ^C'Vitejte v programu "Fotbal ''99"'^M''
   + ^M^C'Hodne uspechu pri hre'^M^C'Vam preji autori:'^M^M^C'Jan Stanek'^M^C'Martin Sip'^M^C'Josef Machaty');
end;

function TAplikace.VytvorNastaveni: POkno;
var
   Okno: POkno;
   R: TRect;
   Prepinace: array [1..3] of PRadioButtons;
   I: Byte;
   Radky: array [1..6] of PInputLine;
begin
   Okno := New (POkno, Init (52, 17, 'Nastaveni hry'));
   with Okno^ do begin
      HelpCtx := hcNastaveniHry;
      R.Assign (3, 3, 29, 6);
      Prepinace [1] := New (PRadioButtons, Init (R,
         NewSItem ('C~l~ovek proti PC',
         NewSItem ('Clove~k~ versus clovek',
         NewSItem ('P~C~ proti PC',
         nil)))
      ));
      Insert (Prepinace [1]);
      for I := 1 to 2 do begin
         R.Assign (3, 8 + (I - 1) * 3, 29, 8 + (I - 1) * 3 + 1);
         Radky [I] := New (PInputLine, Init (R, 24));
         Insert (Radky [I]);
      end;
      R.Assign (3, 14, 29, 15);
      Prepinace [2] := New (PRadioButtons, Init (R,
         NewSItem ('~P~rvni',
         NewSItem ('~D~ruhy',
         nil))
      ));
      Insert (Prepinace [2]);
      R.Assign (31, 3, 49, 5);
      Prepinace [3] := New (PRadioButtons, Init (R,
         NewSItem ('Obdel~n~ik',
         NewSItem ('~H~vezda',
         nil))
      ));
      Insert (Prepinace [3]);
      for I := 1 to 4 do begin
         R.Assign (44, 6 + (I - 1) * 2, 49, 6 + (I - 1) * 2 + 1);
         Radky [I + 2] := New (PInputLine, Init (R, 3));
         Insert (Radky [I + 2]);
      end;
      R.Assign (2, 2, 12, 3);
      Insert (New (PLabel, Init (R, 'Obsazeni', Prepinace [1])));
      R.Move (0, 5);
      Inc (R.B.X, 6);
      Insert (New (PLabel, Init (R, 'Jmeno ~1~. hrace', Radky [1])));
      R.Move (0, 3);
      Insert (New (PLabel, Init (R, 'Jmeno ~2~. hrace', Radky [2])));
      R.Move (0, 3);
      Dec (R.B.X, 2);
      Insert (New (PLabel, Init (R, 'Zacinat bude', Prepinace [2])));
      R.Assign (30, 2, 43, 3);
      Insert (New (PLabel, Init (R, 'Tvar hriste', Prepinace [3])));
      R.Move (0, 4);
      Inc (R.B.X);
      Insert (New (PLabel, Init (R, 'Delk~a~ hriste', Radky [3])));
      R.Move (0, 2);
      Insert (New (PLabel, Init (R, '~V~yska hriste', Radky [4])));
      R.Move (0, 2);
      Dec (R.B.X);
      Insert (New (PLabel, Init (R, 'Delka ~b~rany', Radky [5])));
      R.Move (0, 2);
      Dec (R.B.X);
      Insert (New (PLabel, Init (R, '~Z~pomaleni', Radky [6])));
      R.Assign (30, 14, 40, 16);
      Insert (New (PButton, Init (R, 'S~t~art!', cmOK, bfDefault)));
      R.Move (10, 0);
      Insert (New (PButton, Init (R, '~S~torno', cmCancel, bfNormal)));
      SelectNext (False);
   end;
   VytvorNastaveni := Okno;
end;

function TAplikace.Test: Boolean;
var
   Vysledek: Boolean;
   I: Byte;
   Cislo1, Cislo2, Kod: Integer;
begin
   Vysledek := True;
   with Data do begin
      for I := 1 to 3 do begin
         Val (Zbytek [I], Cislo1, Kod);
         if not ((Kod = 0) and (Cislo1 > 2) and (Cislo1 < 256)) then if Vysledek = True then Vysledek := False;
      end;
      if Tvar = 1 then begin
         Val (Zbytek [1], Cislo1, Kod);
         if not (Odd (Cislo1) and (Zbytek [1] = Zbytek [2])) then if Vysledek = True then Vysledek := False;
      end;
      Val (Zbytek [1], Cislo1, Kod);
      Val (Zbytek [3], Cislo2, Kod);
      if Tvar = 0 then if Cislo2 > Cislo1 then if Vysledek = True then Vysledek := False;
      if Tvar = 1 then if Cislo2 > (Cislo1 div 2) then if Vysledek = True then Vysledek := False;
      Val (Zbytek [4], Cislo1, Kod);
      if not ((Cislo1 >= 0) and (Kod = 0)) then if Vysledek = True then Vysledek := False;
   end;
   Test := Vysledek;
end;

procedure TAplikace.Nastaveni;
var
   StaraData: TData;
   Vysledek: Boolean;
begin
   StaraData := Data;
   repeat
      Vysledek := True;
      if ExecuteDialog (VytvorNastaveni, @Data) = cmOK then begin
         Vysledek := Test;
         if Vysledek = False then Informace (55, 12, 'Chyba v zadani', '1. Rozsah pro parametry hriste a brany je 3..255!'
         + ^M'2. Je-li hriste "Hvezda", pak parametry'^M'   "Delka hriste" a "Vyska hriste" musi byt' + ^M'   shodna licha '
         + 'cisla!'^M'3. "Delka brany" nesmi byt vetsi nez hriste!'^M'4. "Zpomaleni" je kladne cislo!') else Start;
      end else Data := StaraData;
   until Vysledek = True;
end;

procedure TAplikace.NastavParametry;
var I: Integer;
begin
   with Parametry do begin
      with Data do begin
         Val (Zbytek [1], Delka, I);
         Val (Zbytek [2], Vyska, I);
         Val (Zbytek [3], DelkaBrany, I);
         Jmeno1 := Jmena [1];
         Jmeno2 := Jmena [2];
      end;
      Obsazeni := Data.Obsazeni;
      Zacina := Data.Zacina;
      Tvar := Data.Tvar;
      Val (Data.Zbytek [4], Zpomaleni, I);
   end;
end;

procedure TAplikace.Start;
var Vysledek: Boolean;
const
   Ovladac: Integer = VGA;
   Rozliseni: Integer = VGAHi;
begin
   DoneSysError;
   DoneEvents;
   DoneVideo;
   DoneMemory;
   InitGraph (Ovladac, Rozliseni, '');
   if GraphResult = grOK then begin
      Vysledek := True;
      NastavParametry;
      with Parametry do UGrafika.Start (Delka, Vyska, Jmeno1, Jmeno2, Obsazeni, Zacina, Tvar, DelkaBrany, Zpomaleni);
      CloseGraph;
   end else Vysledek := False;
   InitSysError;
   InitEvents;
   InitVideo;
   InitMemory;
   Redraw;
   if Vysledek = False then Informace (37, 7, 'Chyba', 'Chyba pri inicializaci grafiky!');
end;

begin {Hlavni program}
   with Aplikace do begin
      Init;
      Run;
      Done;
   end;
end.