program LZW; { provadi lzw komprimaci }

const
  IC = 256; { je treba inicializovat tabulku }
  EC = 257; { konec souboru }
  FF = 258; { prvni volny }
  SIZE = 4096; { velikost tabulky pro 12 bitu }
  BADI = -1; { neplatny index }

type
  TABLEITEM = record
    val: Byte; { hodnota znaku }
    next: Integer; { dalsi se stejnym prefixem }
    prto: Integer; { prvni, ke ktremu je prefixem }
  end;
  TABLE = array[0 .. SIZE - 1] of TABLEITEM;

var
  i,o: file of Byte; { vstupni a vystupni soubor }
  buf: Byte; { vystupni bitovy buffer }
  buflen: Integer; { pocet bitu v buf }
  tab: TABLE; { komprimacni tabulka }
  firstfree: Integer; { prvni volna polozka v tabulce }
  position: Integer; { aktualni pozice v tabulce }
  width: Integer; { sirka kodu v bitech }
  state_fst: Boolean; { indikator prvniho kodovaneho znaku }
  c: Byte;

procedure BitOutputNew(name: string);
{ name - jmeno vystupniho souboru }
begin
  Assign(o, name);
  Rewrite(o);
  buflen := 0;
end;

procedure BitOutputDestroy;
begin
  if buflen > 0 then
  begin
    buf := buf AND ($FF SHR (8 - buflen));
    Write(o, buf);
  end;
  Close(o);
end;

procedure BitOutputWrite(w: Word; n: Integer);
{ zapise na vystup n bitu z parametru w }
var
  x: Longint;
  xlen: Integer;
begin
  x := buf AND ($FF SHR (8 - buflen));
  x := x OR (Longint(w) SHL buflen);
  xlen := buflen + n;
  while xlen >= 8 do
  begin
    Write(o, Byte(x));
    x := x SHR 8;
    xlen := xlen - 8;
  end;
  buf := Byte(x);
  buflen := xlen;
end;

procedure EncoderInitTable;
{ inicializace tabulky }
var
  i: Integer;
begin
  firstfree := FF;
  width := 9;
  for i := 0 to IC - 1 do
  begin
    tab[i].val := i;
    tab[i].next := BADI;
    tab[i].prto := BADI;
  end;
end;

procedure EncoderPutValue(value: Byte);
{ zakoduje hodnotu value }
var
  newpos, prevpos: Integer;
begin
  if state_fst = True then
  begin
    state_fst := False;
    BitOutputWrite(position, width);
    position := value;
    Exit;
  end;
  newpos := tab[position].prto;
  prevpos := BADI;
  while newpos <> BADI do
  begin
    if tab[newpos].val = value then
    begin
      position := newpos;
      Exit;
    end;
    prevpos := newpos;
    newpos := tab[newpos].next;
  end;
  BitOutputWrite(position, width);
  if firstfree >= SIZE then
  begin
    BitOutputWrite(IC, width);
    EncoderInitTable;
  end
  else
  begin
    tab[firstfree].val := value;
    tab[firstfree].next := BADI;
    tab[firstfree].prto := BADI;
    if tab[position].prto <> BADI then
      tab[prevpos].next := firstfree
    else
      tab[position].prto := firstfree;
    Inc(firstfree);
    if firstfree > 1 SHL width then
      Inc(width);
  end;
  position := value;
end;


procedure EncoderNew;
begin
  EncoderInitTable;
  state_fst := True;
  position := IC;
end;

procedure EncoderDestroy;
begin
  BitOutputWrite(position, width);
  BitOutputWrite(EC, width);
end;

begin
  Assign(i, ParamStr(1));
  Reset(i);
  BitOutputNew(ParamStr(2));
  EncoderNew;
  while not Eof(i) do
  begin
    Read(i, c);
    EncoderPutValue(c);
  end;
  EncoderDestroy;
  BitOutputDestroy;
  Close(i);
end.
