program WZL; { provadi lzw dekomprimaci }

const
  IC = 256; { je treba inicializovat tabulku }
  EC = 257; { konec souboru }
  FF = 258; { prvni volny }
  SIZE = 4096; { velikost tabulky pro 12 bitu }
  BADI = -1; { neplatny index }

type
  TABLEITEM = record
    val: Byte; { hodnota znaku }
    prev: Integer; { predchazejici clen retezce }
  end;
  TABLE = array[0 .. SIZE - 1] of TABLEITEM;

var
  i, o: file of Byte; { vstupni a vystupni soubor }
  buf: Byte; { vystupni bitovy buffer }
  buflen: Integer; { pocet bitu v buf }
  tab: TABLE; { komprimacni tabulka }
  firstfree: Integer; { prvni volna polozka v tabulce }
  width: Integer; { sirka kodu v bitech }
  stack: array [0 .. SIZE - 1] of Byte; { vystupni buffer }
  sp: Integer; { pozice ve vystupnim bufferu }
  firstval: Byte; { hodnota prvniho znaku v dekodovanem retezci }
  code: Word; { aktualni kod }
  oldcode: Word; { predchazejici kod }
  state_add: Boolean; { indikator zda je mozno pridavat do tabulky }
  state_err: Boolean; { indikator chyby, konce souboru, EC }
  c: Byte;

procedure BitInputNew(name: string);
{ name - jmeno vstupniho souboru }
begin
  Assign(i, name);
  Reset(i);
  buflen := 0;
end;

procedure BitInputDestroy;
begin
  Close(i);
end;

function BitInputRead(var w: Word; n: Integer): Boolean;
{ ze vstupu precte n bitu a zapise je promenne w }
var
  x: Longint;
  xlen: Integer;
  t: Byte;
begin
  x := buf AND ($FF SHR (8 - buflen)); { nechame je platne bity }
  xlen := buflen;
  while n > xlen do
  begin
    if Eof(i) then
    begin
      BitInputRead := True;
      Exit;
    end;
    Read(i, t);
    x := x OR (Longint(t) SHL xlen);
    xlen := xlen + 8;
  end;
  buf := x SHR n;
  buflen := xlen - n;
  w := Word(x) AND ($FFFF SHR (16 - n)); { nechame jen platne bity }
  BitInputRead := False;
end;

procedure DecoderInitTable;
{ inicializace tabulky }
var
  i: Integer;
begin
  firstfree := FF;
  width := 9;
  for i := 0 to IC - 1 do
  begin
    tab[i].val := i;
    tab[i].prev := BADI;
  end;
  sp := SIZE;
  state_add := False;
end;

function DecoderReadCode: Boolean;
{ prevede kod na retezec nebo IC, EC }
var
  i: Integer;
begin
  repeat
    oldcode := code;
    if BitInputRead(code, width) = True then
    begin
      DecoderReadCode:= False;
      Exit;
    end;
    if code = EC then
    begin
      state_err := True;
      DecoderReadCode:= False;
      Exit;
    end;
    if code = IC then
      DecoderInitTable;
  until code <> IC;
  sp := SIZE;
  if code < firstfree then
    i := code
  else
  begin
    Dec(sp);
    stack[sp] := firstval;
    i := oldcode;
  end;
  while i <> BADI do
  begin
    Dec(sp);
    stack[sp] := tab[i].val;
    i := tab[i].prev;
  end;
  firstval := stack[sp];
  DecoderReadCode:= True;
end;

procedure DecoderAddItem;
{ prida polozku do tabulky }
begin
  tab[firstfree].val := firstval;
  tab[firstfree].prev := oldcode;
  Inc(firstfree);
  if firstfree >= (1 SHL width) then
    Inc(width);
  if firstfree >= SIZE then
    Dec(width);
end;


procedure DecoderNew;
begin
  state_err := False;
  DecoderInitTable;
end;

function DecoderGetValue(var value: Byte): Boolean;
{ dekoduje znak a ulozi ho do promenne value
  vraci: True - vporadku, False - konec souboru }
begin
  if state_err = True then
  begin
    DecoderGetValue := False;
    Exit;
  end;
  repeat
    if sp < SIZE then
    begin
      state_add := True;
      value := stack[sp];
      Inc(sp);
      DecoderGetValue := True;
      Exit;
    end;
    if DecoderReadCode = False then
    begin
      DecoderGetValue := False;
      Exit;
    end;
    if firstfree >= SIZE then
    begin
      state_err := True;
      DecoderGetValue := False;
      Exit;
    end;
  until state_add = True;
  DecoderAddItem;
  value := stack[sp];
  Inc(sp);
  DecoderGetValue := True;
end;

begin
  Assign(o, ParamStr(2));
  Rewrite(o);
  BitInputNew(ParamStr(1));
  DecoderNew;
  while DecoderGetValue(c) do
    Write(o, c);
  BitInputDestroy;
  Close(o);
end.
