#include "decoder.hpp"


Decoder::Decoder(BitInput *i)
{
  _file = i;
  state_err = 0;
  InitTable();
}

void Decoder::InitTable()
{
  int i;
  firstfree = FF;
  width = 9;
  for (i = 0; i < IC; i++)
  {
    tab[i].val = i;
    tab[i].prev = BADI;
  }
  sp = SIZE;
  state_add = 0;
}

int Decoder::GetValue(uchar &value)
{
  if (state_err != 0)
    return 0;
  do
  {
    if (sp < SIZE)
    {
      state_add = 1;
      value = stack[sp];
      sp++;
      return 1;
    }
    if (ReadCode() == 0)
      return 0;
    if (firstfree >= SIZE)
    {
      state_err = 1;
      return 0;
    }
  }
  while (state_add == 0);
  AddItem();
  value = stack[sp];
  sp++;
  return 1;
}

void Decoder::AddItem()
{
  tab[firstfree].val = firstval;
  tab[firstfree].prev = oldcode;
  firstfree++;
  if (firstfree >= (1 << width))
    width++;
  if (firstfree >= SIZE)
    width--;
}

int Decoder::ReadCode()
{
  int i;
  do
  {
    oldcode = code;
    if (_file->Read(code, width) != 0)
      return 0;
    if (code == EC)
    {
      state_err = 1;
      return 0;
    }
    if (code == IC)
      InitTable();
  }
  while (code == IC);
  sp = SIZE;
  if (code < firstfree)
    i = code;
  else
  {
    sp--;
    stack[sp] = firstval;
    i = oldcode;
  }
  while (i != BADI)
  {
    sp--;
    stack[sp] = tab[i].val;
    i = tab[i].prev;
  }
  firstval = stack[sp];
  return 1;
}
